function hop_demo (tno);
% HOP_DEMO(TNO)
% Demo of simple Hopfield Net - trained on 4 images of 4*4 chars
% Test set is 7 images (the first 4 being the training set)
% Function trains a hopfield net and displays the weights
% It then passes test(tno) to the net and lets it run
% printing the result. tno can be an array of numbers.
% Dr Richard Mitchell, 14/08/03
data{1}=hop(['0110'; '1001'; '1111'; '1001']);   %A
data{2}=hop(['1000'; '1110'; '1010'; '1110']);   %B
data{3}=hop(['0110'; '1000'; '1000'; '0110']);   %C
data{4}=hop(['1110'; '0101'; '0101'; '1110']);   %D

dd=[]; for ct=1:4, dd=[dd, dispnet(data{ct})]; end
dd(dd=='0')='.';                    % 0 -> . so easier to see
disp(dd), fprintf('\n');

test=data;
test{5}=hop(['0110'; '1000'; '1010'; '1000']);
test{6}=hop(['1000'; '1000'; '1000'; '1111']);
test{7}=hop(['0001'; '0110'; '1000'; '0110']);
if nargin==0, tno=1; end

netsize = 16;                               % 16 nodes ....
nodes = zeros(netsize,1);                   % 16 outputs
weights = hop_init2(data, netsize);          % initialise weights
disp(num2str(weights, '%3d')), fprintf('\n');

for tct=1:length(tno)                       % present test inputs
    if tno(tct)>0 & tno(tct)<=length(test)
        nodes = test{tno(tct)};              % load nodes with input
        energy = hop_energy(nodes, weights);
		dout = dispnet(nodes);                  % store image of output
		for epoch=1:8
            [nodes, changed] = hop_iterate(nodes, weights);% update
            if changed, 
                dout = [dout, dispnet(nodes)];  
                energy(epoch+1) = hop_energy(nodes, weights);
            end    % add to image of output
		end
        dout(dout=='0')='.';                    % 0 -> . so easier to see
		disp(dout), fprintf('\n');
        disp(energy),fprintf('\n');
    end
end

function weights = hop_init(data, netsize); % calc weights of network
weights = zeros (netsize, netsize);         % zero weights
for i = 1:netsize                           % set value by nested for loops
    for j = 1:netsize
        if i ~= j                           % on diagonal, weights 0
            weights(i,j) = 0;               % initialise sum
            for d=1:length(data)            % for each in training set
                weights(i,j) = weights(i,j) + data{d}(i) * data{d}(j);
            end                             % add prod of ith and jth value
        end
    end
end

function weights = hop_init2(data, netsize);% alternative
x = [];           % x comprises one col per data set
for d = 1:length(data),                     % for each data item
   x = [x, data{d}];                        % add column to x
end
weights = x*x' - eye(netsize)*length(data); % compute weights

function [newnodes, change] = hop_iterate(nodes, weights);    % calc next output
newnodes = nodes;                           % copy current output
for ct = 1:length(nodes)                    % for each node
    v = 0;
    for oct = 1:length(nodes), v = v + weights(ct, oct) * nodes(oct); end
                                            % sum weight * other node
    if v > 0, newnodes(ct) = 1; elseif v < 0 newnodes(ct) = -1; end
end                 % generate new node output (or leave unchanged)
change = sum(newnodes ~= nodes);                 % has it changed?

function energy = hop_energy (nodes, weights);
energy = 0;
for i=1:length(nodes)
    for j=1:length(nodes)
        energy = energy + weights(i,j)*nodes(i)*nodes(j);
    end
end
energy = -0.5 * energy;

function val = hop(data);                   % convert 4*4 '0'/'1'char array to col of -1/1
val = ones(prod(size(data)),1);
for ct=1:prod(size(data))
    if data(ct)=='0', val(ct) = -1; end
end

function dout = dispnet(nodes)              
% output nodes as 4*4 chars, preceded by two spaces
dout = [[' ';' ';' ';' '], num2str(reshape((nodes+1)/2, 4, 4),'%1d')];
