function net = kn_example (data);
% NET = KN_EXAMPLE(DATA) - demo of Kohonen Network on DATA
% DATA is a matrix, last col being output class rest are inputs
% if not supplied, test DATA generated randomly
% Each node in NET has n weights where n is number of inputs.
% The algorithm runs over 9 epochs, presenting the DATA many times
% After each epoch, the NET is plotted where an attempt is made
% to classify each node - using kn_class
% Dr Richard Mitchell, 12/8/03
clf;
if nargin==0
    c=makecluster([10,15,12,8], [0.2 0.1 0.2 0.3], [0.3 0.5 0.8; 0.2 0.5 0.2; 0.7 0.3 0.6; 0.5 0.4 0.3]);
    data=[]; for ct=1:4, data=[data; c{ct}, ct+zeros(size(c{ct},1),1)]; end
end
ins = data(:,[1:size(data,2)-1]);   % get all inputs (last col of data is output)
net = rand(10,8,size(ins,2));       % net is 10 rows 8 cols, 1 weight per input var
neigh = [6 5 4 3 2 1 1 1];          % the neighbourhood at each (of 8) epoch
lrate = [0.5 0.4 0.3 0.2 0.1 0.05 0.03 0.02 ]; % the learning rate at each epoch

for epoch = 1:length(neigh)
    draw(net, data, epoch)          % show current weights
    for ect=1:40                    % present input 40 times
        for ict = 1:size(ins, 1)    % 'learn' each input pair
            net = kn_learn(net, ins(ict,:), neigh(epoch), lrate(epoch));
        end
    end
end
draw(net, data, length(neigh)+1);               % show final state of net

function draw(net, data, epoch)
% draw network at its current state
subplot(3,3,epoch);         % set appropriate subplot
hold on                     % so can keep drawing lines
col={'y*', 'w*', 'r*', 'b*' , 'g*', 'm*'};
ncol = 2 + kn_class(net, data, 1);
for r = 1:size(net, 1)    % for all rows, and columns
    for c = 1:size(net, 2)
         plot(r, c, col{ncol(r,c)});
    end
end
set(gca, 'xlim', [0 size(net,1)+1], 'xtick', [], ...
         'ylim', [0 size(net,2)+1], 'ytick', []);
hold off
