function [mlp, tserr]=mlp_learn(mlp, invals, outvals, paras);
% [MLP, TSERR] = MLP_LEARN (MLP, INVALS, OUTVALS, PARAS);
% for specified INVALS and expected OUTVALS, adjust weights using PARAS
% each row in INVALS and corresponding row in OUTVALS is an item in training set
% TSERR is sum of square of errors in whole training set
% PARAS = [Learnrate]       % momentum could be added as second para
% Dr Richard Mitchell 24.7.03
tserr = 0;
for ct=1:size(invals,1)                     % for each item in training set
    [mlp, tse] = do_learn (mlp, invals(ct,:), outvals(ct, :), paras);
    tserr = tserr + tse;
end

function [mlp, tserr] = do_learn(mlp, invals, outvals, paras);
tserr = 0;
mlp = mlp_calcoutputs (mlp, invals);
oct = length(outvals);
for ct=length(mlp):-1:length(invals)+1,     % for all output then hidden nodes
    [mlp(ct), tserr, oct] = calcdelta (mlp, mlp(ct), tserr, outvals, oct);
                                            % calc actual delta
    wins = 1;                               % find 'input' of all weights, for bias = 1
    for wct = 2:length(mlp(ct).innodes), wins = [wins, mlp(mlp(ct).innodes(wct)).output]; end
                                            % next calculate change in all weights
    mlp(ct).dweights = mlp(ct).delta  * paras(1) * wins;
end
for ct = length(invals)+1:length(mlp)       % now change all weights by calculated amount
    mlp(ct).weights = mlp(ct).weights + mlp(ct).dweights;
end

function [node, tserr, oct] = calcdelta (mlp, node, tserr, outvals, oct);     
% calc delta of output/hidden node
if node.type==3                         % is output node
    delest = outvals(oct) - node.output;
    oct = oct - 1;                      % next time is previous output
    tserr = tserr + delest^2;           % add square of error at output
else                                    % is hidden node
    delest = 0;                         % some weighted deltas of nodes in next layer
    for ct2 = 1:size(node.goesto, 2)
        onode = node.goesto(:,ct2);     % get node goes to, and assoc weight number
        delest = delest + mlp(onode(1)).delta * mlp(onode(1)).weights(onode(2));
    end
end
node.delta = node.output * (1 - node.output) * delest;
