function [rbfn, yt, yu] = rbf_demo (train, unseen, cndx);
% [RBFN, YT, YU] = RBF_DEMO (TRAIN, UNSEEN, [CNDX]);
% Demo of RBF - TRAIN & UNSEEN are matrices, last col is output, rest are inputs
% RBFN is set up and trained on TRAIN data and then tested on UNSEEN
% YT are the outputs of the trained net on TRAIN; YU outputs of UNSEEN
% Routine plots the actual and calculated outputs as TADPOLE plots
% CNDX(n)=r means element n in training set is in cluster r; and the r
% clusters are used for the centres of the gaussian basis functions
% if CNDX is not provided, kmeans is used to find it.
% Test data that can be used, type
%   >> load demo_clu.mat
%   >> rbf_demo(demo_clutrain.vector,demo_cluunseen.vector)
% Dr Richard Mitchell ..15/08/03
if nargin < 3
    cndx = kmeans(train(:,size(train,2)-1), 10, [3:20]);
end

[rbfn, yt, yu, terr, uerr] = rbf_make (train, unseen, cndx);
alldata = [train(:,size(train,2)); unseen(:,size(train,2)); yt; yu];
clf; 
subplot(2,1,1);
%vis_model(train(:,3), yt, 'tadpole', 'r.', 'b-');
tadpole(train(:,size(train,2)), yt);
set(gca,'ylim', [floor(min(alldata)), ceil(max(alldata))]);
subplot(2,1,2);
tadpole(unseen(:,size(train,2)), yu);
%vis_model(unseen(:,3), yu, 'tadpole', 'r.', 'b-')
set(gca,'ylim', [floor(min(alldata)), ceil(max(alldata))]);
fig_set('lines', 'linewidth', 2);
fig_set('text', 'fontsize', 16);
fig_set('dots', 'markersize', 16)

fprintf('Training error is %4.2f, Unseen error %4.2f', terr, uerr);

