function [rbfn, yt, yu, terr, uerr] = rbf_make (train, unseen, clusterindex);
% [RBFN, YT, YU, TERR, UERR] = RBF_MAKE (TRAIN, UNSEEN, CLUSTERINDEX)
% Make RBFN train it on TRAIN set, test it on UNSEEN set
% TRAIN/UNSEEN are matrices : last col is output, earlier cols are inputs
% CLUSTERINDEX(r) = n means rth item in TRAIN set is in cluster n
% and this is used to determine centres of RBF basis functions
% YT are outputs for trained RBFN when inputs are TRAIN
% YU are outputs to trained RBFN when inputs are UNSEEN
% TERR and UERR are mean sum squared error
% Dr Richard Mitchell ..15/08/03
outcol = size(train,2);
for ct=1:max(clusterindex), 
    if isempty(ct==unique(clusterindex))
        cc(ct,[1:outcol-1]) = 0;
    else
        cc(ct,:) = mean(train(clusterindex==ct, [1:outcol-1]), 1); 
    end
end
rad = findrad (clusterindex, train, cc, outcol);
rbfn=struct('centres', cc', ...
            'radius', rad, ...     
            'weights', zeros(1,1+max(clusterindex)));
for ct = 1:size(train,1)
    [dummy, h(ct,:)] = rbf_calc(rbfn, train(ct, [1:outcol-1]));
end
rbfn.weights = h\train(:,outcol);
for ct = 1:size(train, 1)
    yt(ct,1) = rbf_calc(rbfn, train(ct, [1:outcol-1]));
end
terr = sum ((yt - train(:,outcol)).^2) / size(train, 1);
for ct = 1:size(unseen, 1)
    yu(ct,1) = rbf_calc(rbfn, unseen(ct, [1:outcol-1]));
end
uerr = sum ((yu - unseen(:,outcol)).^2) / size(unseen, 1);

function rad = findrad (cndx, train, cc, outcol);
% simply rad is 4* that of circle round cluster centre which includes all points
for ct=1:max(cndx), 
    incl = train(cndx==ct, [1:outcol-1]);
    for ct2 = 1:outcol-1,  incl(:,ct2) = incl(:,ct2) - cc(ct,ct2); end
    if isempty(incl), rad(ct) = 0.1; else rad(ct) = 4*max(dot(incl, incl, 2)); end
end

