function ans = wnn_simple (data, tuplemap, tsize, discrim)
% Used to Learn a set of data into a weightless neural network
% or to see how a WNN responds to a data set
% To learn invoke by
%   DISCRIM = WNN_SIMPLE(DATA, TUPLEMAP, TSIZE)
%       DATA is cell array of data (each typically 8*8 chars)
%       TUPLEMAP specifies the order in which it is sampled
%       TSIZE is tuplesize
%       This routine returns in DISCRIM the taught image(s)
% or analyse by
%   Count = WNN_SIMPLE(DATA, TUPLEMAP, TSIZE, DISCRIM)
%       DATA is one item or a cell array of data (each typically 8*8 chars)
%       TUPLEMAP specifies the order in which it is sampled
%       TSIZE is tuplesize
%       DISCRIM is the taught network
%       Routine returns in Count the % of firing neurons
% Dr Richard Mitchell 29.7.03
if ~iscell(data), data = {data}; end        % single data is 1 element array
if nargin < 4           % if learning
    discrim = www_zero (data, tsize);
    for ct = 1:length(data)
       discrim = www_learn (discrim, data{ct}, tuplemap, tsize);
    end
    ans = discrim;
else
    for ct = 1:length(data), 
        ans(ct) = www_analyse(discrim, data{ct}, tuplemap, tsize);
    end
    ans = round(100 * ans / size(discrim, 2));
end
    
function discrim = www_zero (data, tsize);
if iscell(data), dsize = size(data{1}); else dsize = size(data); end
numrams = prod(dsize) / tsize;
discrim = zeros(2^tsize, numrams);      % each column is a RAM

function discrim = www_learn(discrim, data, tuplemap, tsize);
tct = 1;
for ram = 1:size(discrim, 2)             % for all RAMs
    [tuple, tct] = www_gettuple (data, tuplemap, tct, tsize);
    discrim(tuple, ram) = 1;
end

function fires = www_analyse(discrim, data, tuplemap, tsize);
fires = 0;
tct = 1;
for ram = 1:size(discrim, 2)             % for all RAMs
    [tuple, tct] = www_gettuple (data, tuplemap, tct, tsize);
    if discrim(tuple, ram) == 1, fires = fires + 1; end
end

function [tuple, tct] = www_gettuple (data, tuplemap, tct, tsize);
tuple = 0;
for ct = 1:tsize
    tuple = tuple * 2 + (data(tuplemap(tct)) == '1');
    tct = tct + 1;
end
tuple = tuple + 1;                  % need 1..n not 0..n-1

